@@ -2,7 +2,7 @@
 
 ;; Copyright (C) 2015-2017  Jorgen Schaefer <contact@jorgenschaefer.de>
 
-;; Version: 1.19
+;; Version: 1.22
 ;; Author: Jorgen Schaefer <contact@jorgenschaefer.de>
 ;; Package-Requires: ((emacs "24.3"))
 ;; URL: https://github.com/jorgenschaefer/emacs-buttercup
@@ -123,14 +123,14 @@ a call to `save-match-data', as `format-spec' modifies that."
 This macro knows three forms:
 
 \(expect ARG :MATCHER ARGS...)
-  Fail the current test iff the matcher does not match these arguments.
+  Fail the current test if the matcher does not match these arguments.
   See `buttercup-define-matcher' for more information on matchers.
 
 \(expect (function ARG...))
-  Fail the current test iff the function call does not return a true value.
+  Fail the current test if the function call does not return a true value.
 
 \(expect ARG)
-  Fail the current test iff ARG is not true."
+  Fail the current test if ARG is not true."
   (let ((wrapped-args
          (mapcar (lambda (expr) `(lambda () (quote ,expr) ,expr)) args)))
     `(buttercup-expect
@@ -179,7 +179,7 @@ FORMAT and ARGS are passed to `format'."
   (signal 'buttercup-pending (apply #'format format args)))
 
 (defmacro assume (condition &optional message)
-  "Assume CONDITIION for the current test.
+  "Assume CONDITION for the current test.
 
 Assume that CONDITION evaluates to non-nil in the current test.
 If it evaluates to nil cancel the current test with MESSAGE. If
@@ -205,11 +205,13 @@ failed."
           ,@body)))
 
 (defun buttercup--function-as-matcher (fun)
+  "Wrap FUN in code to unpack function-wrapped arguments."
   (cl-assert (functionp fun) t)
   (lambda (&rest args)
     (apply fun (mapcar #'funcall args))))
 
 (defun buttercup--find-matcher-function (matcher)
+  "Return the matcher function for MATCHER."
   (let ((matcher-prop
          (when (symbolp matcher)
            (get matcher 'buttercup-matcher))))
@@ -235,7 +237,6 @@ failed."
 
 ARGS is a list of functions that must be `funcall'ed to get their
 values.
-
 MATCHER is either a matcher keyword defined with
 `buttercup-define-matcher', or a function."
   (cl-assert (cl-every #'buttercup--wrapper-fun-p args) t)
@@ -244,19 +245,20 @@ MATCHER is either a matcher keyword defined with
     (apply function args)))
 
 (cl-defmacro buttercup--test-expectation
-    (expr &key expect-match-phrase expect-mismatch-phrase)
+    (expression &key expect-match-phrase expect-mismatch-phrase)
   "Wrapper for the common matcher case of two possible messages.
 
 The logic for the return values of buttercup matchers can be
 unintuitive, since the return value is a cons cell whose first
 element is t for a mismatch and nil for a match. In the simple
-case where there are only two possible messages (one for a match
-and one for a mismatch), this macro allows you to simply specify
-those two phrases and the expression to test."
+case where there are only two possible
+messages (EXPECT-MATCH-PHRASE for a match and
+EXPECT-MISMATCH-PHRASE for a mismatch), this macro allows you to
+simply specify those two phrases and the EXPRESSION to test."
   (declare (indent 1))
   (cl-assert expect-match-phrase)
   (cl-assert expect-mismatch-phrase)
-  `(let ((value ,expr))
+  `(let ((value ,expression))
      (if value
          (cons t ,expect-mismatch-phrase)
        (cons nil ,expect-match-phrase))))
@@ -264,7 +266,7 @@ those two phrases and the expression to test."
 (cl-defmacro buttercup-define-matcher-for-unary-function
     (matcher function &key
              expect-match-phrase expect-mismatch-phrase function-name)
-  "Shortcut to define a macther for a 1-argument function.
+  "Shortcut to define a MATCHER for a 1-argument FUNCTION.
 
 When the matcher is used, keyword arguments EXPECT-MATCH-PHRASE
 and EXPECT-MISMATCH-PHRASE are used to construct the return
@@ -338,7 +340,7 @@ See also `buttercup-define-matcher'."
 (cl-defmacro buttercup-define-matcher-for-binary-function
     (matcher function &key
              expect-match-phrase expect-mismatch-phrase function-name)
-  "Shortcut to define a macther for a 2-argument function.
+  "Shortcut to define a MATCHER for a 2-argument FUNCTION.
 
 When the matcher is used, keyword arguments EXPECT-MATCH-PHRASE
 and EXPECT-MISMATCH-PHRASE are used to construct the return
@@ -647,7 +649,7 @@ See also `buttercup-define-matcher'."
   (setq spy (funcall spy)
         number (funcall number))
   (cl-assert (symbolp spy))
-  (let* ((call-count (length (spy-calls-all spy))))
+  (let* ((call-count (spy-calls-count spy)))
     (cond
      ((= number call-count)
       t)
@@ -689,11 +691,13 @@ See also `buttercup-define-matcher'."
   function)
 
 (defun buttercup-suite-add-child (parent child)
-  "Add a CHILD suite to a PARENT suite."
+  "Add a CHILD suite to a PARENT suite.
+Return CHILD."
   (setf (buttercup-suite-children parent)
         (append (buttercup-suite-children parent)
                 (list child)))
-  (setf (buttercup-suite-or-spec-parent child) parent))
+  (setf (buttercup-suite-or-spec-parent child) parent)
+  child)
 
 (defun buttercup-suite-or-spec-parents (suite-or-spec)
   "Return a list of parents of SUITE-OR-SPEC."
@@ -787,6 +791,16 @@ See also `buttercup-define-matcher'."
    (or (buttercup-suite-or-spec-time-ended suite-or-spec) (current-time))
    (or (buttercup-suite-or-spec-time-started suite-or-spec) (current-time))))
 
+(defun buttercup-elapsed-time-string (suite-or-spec)
+  "Convert the elapsed time for SUITE-OR-SPEC to a short string."
+  (seconds-to-string (float-time (buttercup-elapsed-time suite-or-spec))))
+
+(defun buttercup--indented-description (suite-or-spec)
+  "Return the description of SUITE-OR-SPEC indented according to level.
+The indentaion is two spaces per parent."
+  (let ((level (length (buttercup-suite-or-spec-parents suite-or-spec))))
+    (concat (make-string (* 2 level) ?\s) (buttercup-suite-or-spec-description suite-or-spec))))
+
 ;;;;;;;;;;;;;;;;;;;;
 ;;; Suites: describe
 
@@ -843,7 +857,8 @@ is a function containing the body instructions passed to
            (format "Found duplicate spec names in suite: %S"
                    (delete-dups dups)))))
       (setq buttercup-suites (append buttercup-suites
-                                     (list buttercup--current-suite))))))
+                                     (list buttercup--current-suite)))
+      buttercup--current-suite)))
 
 ;;;;;;;;;;;;;
 ;;; Specs: it
@@ -865,7 +880,8 @@ most probably including one or more calls to `expect'."
   "Function to handle an `it' form.
 
 DESCRIPTION has the same meaning as in `it'. BODY-FUNCTION is a
-function containing the body instructions passed to `it'."
+function containing the body instructions passed to `it'. Return
+the created spec object."
   (declare (indent 1))
   (when (not buttercup--current-suite)
     (error "`it' has to be called from within a `describe' form"))
@@ -1003,17 +1019,18 @@ DESCRIPTION is a string. BODY is ignored."
   "Like `buttercup-it', but mark the spec as disabled.
 A disabled spec is not run.
 
-DESCRIPTION has the same meaning as in `xit'. FUNCTION is ignored."
+DESCRIPTION has the same meaning as in `xit'. FUNCTION is
+ignored. Return the created spec object."
   (declare (indent 1))
   (ignore function)
-  (buttercup-it description (lambda ()
-                              (signal 'buttercup-pending "PENDING")))
-  (let ((spec (car (last (buttercup-suite-children
-                          buttercup--current-suite)))))
+  (let ((spec (buttercup-it description
+                (lambda ()
+                  (signal 'buttercup-pending "PENDING")))))
     (setf (buttercup-spec-status spec)
           'pending
           (buttercup-spec-failure-description spec)
-          "")))
+          "")
+    spec))
 
 ;;;;;;;;;
 ;;; Spies
@@ -1022,10 +1039,49 @@ DESCRIPTION has the same meaning as in `xit'. FUNCTION is ignored."
                                                  :weakness 'key)
   "A mapping of currently-defined spies to their contexts.")
 
-(cl-defstruct spy-context
-  args
-  return-value
-  current-buffer)
+;; The base struct has no constructor so a factory function
+;; `make-spy-context' masquerading as a constructor can be defined
+;; later.
+(cl-defstruct (spy-context (:constructor nil))
+  args current-buffer)
+(cl-defstruct (spy-context-return (:include spy-context)
+                                  (:conc-name spy-context--return-))
+  value)
+(cl-defstruct (spy-context-thrown (:include spy-context)
+                                  (:conc-name spy-context--thrown-))
+  signal)
+
+(cl-defun make-spy-context (&key args current-buffer
+                                 (return-value nil has-return-value)
+                                 (thrown-signal nil has-thrown-signal))
+  "Constructor for objects of type spy-context.
+ARGS is the argument list of the called function.
+CURRENT-BUFFER is the buffer that was current when the spy was called.
+RETURN-VALUE is the returned value, if any.
+THROWN-SIGNAL is the signal raised by the function, if any.
+Only one of RETURN-VALUE and THROWN-SIGNAL may be given. Giving
+none of them is equivalent to `:return-value nil'."
+  (cond
+   ((and has-return-value has-thrown-signal)
+    (error "Only one of :return-value and :thrown-signal may be given"))
+   (has-thrown-signal (make-spy-context-thrown :args args
+                                               :current-buffer current-buffer
+                                               :signal thrown-signal))
+   (t (make-spy-context-return :args args
+                               :current-buffer current-buffer
+                               :value return-value))))
+
+(defun spy-context-return-value (context)
+  "Access slot \"return-value\" of `spy-context' struct CONTEXT."
+  (unless (spy-context-return-p context)
+    (error "Not a returning context"))
+  (spy-context--return-value context))
+
+(defun spy-context-thrown-signal (context)
+  "Access slot \"thrown-signal\" of `spy-context' struct CONTEXT."
+  (unless (spy-context-thrown-p context)
+    (error "Not a signal-raising context"))
+  (spy-context--thrown-signal context))
 
 (defun spy-on (symbol &optional keyword arg)
   "Create a spy (mock) for the function SYMBOL.
@@ -1052,10 +1108,10 @@ also be a command with the same interactive form, unless
 `:and-call-fake' is used, in which case it is the caller's
 responsibility to ensure ARG is a command."
   ;; We need to load an autoloaded function before spying on it
-  (when (autoloadp (symbol-function symbol))
+  (when (autoloadp (and (fboundp symbol) (symbol-function symbol)))
     (autoload-do-load (symbol-function symbol) symbol))
-  (cl-assert (not (autoloadp (symbol-function symbol))))
-  (let* ((orig (symbol-function symbol))
+  (cl-assert (not (autoloadp (and (fboundp symbol) (symbol-function symbol)))))
+  (let* ((orig (and (fboundp symbol) (symbol-function symbol)))
          (orig-intform (interactive-form orig))
          (replacement
           (pcase
@@ -1094,36 +1150,59 @@ responsibility to ensure ARG is a command."
                 nil))
             (_
              (error "Invalid `spy-on' keyword: `%S'" keyword)))))
-    (buttercup--spy-on-and-call-fake symbol replacement)))
+    (unless (buttercup--spy-on-and-call-replacement symbol replacement)
+      (error "Spies can only be created in `before-each'"))))
 
-(defun buttercup--spy-on-and-call-fake (spy fake-function)
-  "Replace the function in symbol SPY with a spy calling FAKE-FUNCTION."
-  (let ((orig-function (symbol-function spy)))
-    (fset spy (buttercup--make-spy fake-function))
-    (buttercup--add-cleanup (lambda ()
-                              (fset spy orig-function)))))
 
-(defun buttercup--make-spy (fake-function)
-  "Create a new spy function wrapping FAKE-FUNCTION and tracking calls to itself."
+(defun buttercup--spy-on-and-call-replacement (spy fun)
+  "Replace the function in symbol SPY with a spy calling FUN."
+  (let ((orig-function (and (fboundp spy) (symbol-function spy))))
+    (when (buttercup--add-cleanup (lambda ()
+                                  (fset spy orig-function)))
+      (fset spy (buttercup--make-spy fun)))))
+
+(defun buttercup--make-spy (fun)
+  "Create a new spy function wrapping FUN and tracking every call to itself."
   (let (this-spy-function)
-    (setq this-spy-function
-          (lambda (&rest args)
-            (let ((return-value (apply fake-function args)))
+    (setq
+     this-spy-function
+     (lambda (&rest args)
+       (let ((returned nil)
+             (return-value nil))
+         (condition-case err
+             (progn
+               (setq return-value (apply fun args)
+                     returned t)
+               (buttercup--spy-calls-add
+                this-spy-function
+                (make-spy-context :args args
+                                  :return-value return-value
+                                  :current-buffer (current-buffer)))
+               return-value)
+           (error
+            ;; If returned is non-nil, then the error we caught
+            ;; didn't come from FUN, so we shouldn't record it.
+            (unless returned
               (buttercup--spy-calls-add
                this-spy-function
                (make-spy-context :args args
-                                 :return-value return-value
-                                 :current-buffer (current-buffer)))
-              return-value)))
-    ;; Add the interactive form from `fake-function', if any
-    (when (interactive-form fake-function)
+                                 :thrown-signal err
+                                 :current-buffer (current-buffer))))
+            ;; Regardless, we only caught this error in order to
+            ;; record it, so we need to re-throw it.
+            (signal (car err) (cdr err)))))))
+    ;; Add the interactive form from `fun', if any
+    (when (interactive-form fun)
       (setq this-spy-function
             `(lambda (&rest args)
-               ,(interactive-form fake-function)
+               ,(interactive-form fun)
                (apply ',this-spy-function args))))
     this-spy-function))
 
-(defvar buttercup--cleanup-functions nil)
+(defvar buttercup--cleanup-functions :inactive
+  "Stack of cleanup operations.
+Should always be set to a value that is not `listp', except while
+in a `buttercup-with-cleanup' environment.")
 
 (defmacro buttercup-with-cleanup (&rest body)
   "Execute BODY, cleaning spys and the rest afterwards."
@@ -1135,16 +1214,17 @@ responsibility to ensure ARG is a command."
 
 (defun buttercup--add-cleanup (function)
   "Register FUNCTION for cleanup in `buttercup-with-cleanup'."
-  (setq buttercup--cleanup-functions
-        (cons function buttercup--cleanup-functions)))
+  (when (listp buttercup--cleanup-functions)
+    (setq buttercup--cleanup-functions
+          (cons function buttercup--cleanup-functions))))
 
 (defun spy-calls-all (spy)
-  "Return the contexts of calls to SPY."
+  "Return all call contexts for SPY."
   (gethash (symbol-function spy)
            buttercup--spy-contexts))
 
 (defun buttercup--spy-calls-add (spy-function context)
-  "Add CONTEXT to the recorded calls to SPY."
+  "Add CONTEXT to the call records for SPY-FUNCTION."
   (puthash spy-function
            (append (gethash spy-function
                             buttercup--spy-contexts)
@@ -1152,7 +1232,7 @@ responsibility to ensure ARG is a command."
            buttercup--spy-contexts))
 
 (defun spy-calls-reset (spy)
-  "Reset SPY, removing all recorded calls."
+  "Reset SPY, removing all stored contexts."
   (puthash (symbol-function spy)
            nil
            buttercup--spy-contexts))
@@ -1172,6 +1252,14 @@ responsibility to ensure ARG is a command."
   "Return the number of times SPY has been called so far."
   (length (spy-calls-all spy)))
 
+(defun spy-calls-count-returned (spy)
+  "Return the number of times SPY has been called successfully so far."
+  (cl-count-if 'spy-context-return-p (spy-calls-all spy)))
+
+(defun spy-calls-count-errors (spy)
+  "Return the number of times SPY has been called and thrown errors so far."
+  (cl-count-if 'spy-context-thrown-p (spy-calls-all spy)))
+
 (defun spy-calls-args-for (spy index)
   "Return the context of the INDEXth call to SPY."
   (let ((context (elt (spy-calls-all spy)
@@ -1181,7 +1269,7 @@ responsibility to ensure ARG is a command."
       nil)))
 
 (defun spy-calls-all-args (spy)
-  "Return the arguments to all calls to SPY."
+  "Return the arguments for every recorded call to SPY."
   (mapcar 'spy-context-args (spy-calls-all spy)))
 
 (defun spy-calls-most-recent (spy)
@@ -1290,15 +1378,21 @@ current directory."
         (when (not (string-match "\\(^\\|/\\)\\." (file-relative-name file)))
           (load file nil t))))
     (when patterns
-      (dolist (spec (buttercup--specs buttercup-suites))
-        (let ((spec-full-name (buttercup-spec-full-name spec)))
-          (unless (cl-dolist (p patterns)
-                    (when (string-match p spec-full-name)
-                      (cl-return t)))
-            (setf (buttercup-spec-function spec)
-                  (lambda () (signal 'buttercup-pending "SKIPPED")))))))
+      (buttercup--mark-skipped buttercup-suites patterns))
     (buttercup-run)))
 
+(defun buttercup--mark-skipped (suites patterns)
+  "Mark any spec in SUITES not matching PATTERNS as skipped.
+SUITES is a list of suites. PATTERNS is a list of regexps."
+  (dolist (spec (buttercup--specs suites))
+    (let ((spec-full-name (buttercup-spec-full-name spec)))
+      (unless (cl-dolist (p patterns)
+                (when (string-match p spec-full-name)
+                  (cl-return t)))
+        (setf (buttercup-spec-function spec)
+              (lambda () (signal 'buttercup-pending "SKIPPED"))
+              (buttercup-spec-status spec) 'pending)))))
+
 ;;;###autoload
 (defun buttercup-run-markdown-buffer (&rest markdown-buffers)
   "Run all test suites defined in MARKDOWN-BUFFERS.
@@ -1410,8 +1504,18 @@ Do not change the global value.")
         (with-current-buffer buttercup-warning-buffer-name
           (when (string-match-p "[^[:space:]\n\r]" (buffer-string))
             (buttercup--print
+             "%s\n"
              (buttercup-colorize
-              (buffer-string)
+              ;; Any terminating newline in the buffer should not be
+              ;; colorized. It would mess up color handling in Emacs
+              ;; compilation buffers using
+              ;; `ansi-color-apply-on-region' in
+              ;; `compilation-filter-hook'.
+              (buffer-substring (point-min)
+                                (save-excursion
+                                  (goto-char (1- (point-max)))
+                                  (if (looking-at-p "\n")
+                                      (point) (point-max))))
               'yellow)))))
     (when (get-buffer buttercup-warning-buffer-name)
       (kill-buffer buttercup-warning-buffer-name))
@@ -1432,8 +1536,13 @@ failed and pending specs."
         (`(error (buttercup-pending . ,pending-description))
          (setq status 'pending
                description pending-description))))
-    (when (memq (buttercup-suite-or-spec-status suite-or-spec)
-                '(passed pending))
+    ;; Only change state when the new state is 'worse' than or same as
+    ;; the current state. The constant list is the prioritized list of
+    ;; states. The new state is worse if it is in the tail of the
+    ;; current state.
+    (when (memq status
+                (memq (buttercup-suite-or-spec-status suite-or-spec)
+                      '(passed pending failed)))
       (setf (buttercup-suite-or-spec-status suite-or-spec) status
             (buttercup-suite-or-spec-failure-description suite-or-spec) description
             (buttercup-suite-or-spec-failure-stack suite-or-spec) stack))))
@@ -1468,7 +1577,7 @@ EVENT and ARG are described in `buttercup-reporter'."
         (print-escape-nonascii t))
     (pcase event
       (`buttercup-started
-       (setq buttercup-reporter-batch--start-time (float-time)
+       (setq buttercup-reporter-batch--start-time (current-time)
              buttercup-reporter-batch--failures nil)
        (let ((defined (buttercup-suites-total-specs-defined arg))
              (pending (buttercup-suites-total-specs-pending arg)))
@@ -1479,17 +1588,9 @@ EVENT and ARG are described in `buttercup-reporter'."
            (buttercup--print "Running %s specs.\n\n" defined))))
 
       (`suite-started
-       (let ((level (length (buttercup-suite-or-spec-parents arg))))
-         (buttercup--print "%s%s\n"
-                           (make-string (* 2 level) ?\s)
-                           (buttercup-suite-description arg))))
-
+         (buttercup--print "%s\n" (buttercup--indented-description arg)))
       (`spec-started
-       (let ((level (length (buttercup-suite-or-spec-parents arg))))
-         (buttercup--print "%s%s"
-                           (make-string (* 2 level) ?\s)
-                           (buttercup-spec-description arg))))
-
+         (buttercup--print "%s" (buttercup--indented-description arg)))
       (`spec-done
        (cond
         ((eq (buttercup-spec-status arg) 'passed)) ; do nothing
@@ -1502,9 +1603,7 @@ EVENT and ARG are described in `buttercup-reporter'."
          (buttercup--print "  %s" (buttercup-spec-failure-description arg)))
         (t
          (error "Unknown spec status %s" (buttercup-spec-status arg))))
-       (buttercup--print " (%s)\n"
-                         (seconds-to-string
-                          (float-time (buttercup-elapsed-time arg)))))
+       (buttercup--print " (%s)\n" (buttercup-elapsed-time-string arg)))
 
       (`suite-done
        (when (= 0 (length (buttercup-suite-or-spec-parents arg)))
@@ -1534,19 +1633,20 @@ EVENT and ARG are described in `buttercup-reporter'."
        (let ((defined (buttercup-suites-total-specs-defined arg))
              (pending (buttercup-suites-total-specs-pending arg))
              (failed (buttercup-suites-total-specs-failed arg))
-             (duration (- (float-time)
-                          buttercup-reporter-batch--start-time)))
+             (duration (float-time (time-subtract
+                                    (current-time)
+                                    buttercup-reporter-batch--start-time))))
          (if (> pending 0)
              (buttercup--print
-              "Ran %s out of %s specs, %s failed, in %.1f seconds.\n"
+              "Ran %s out of %s specs, %s failed, in %s.\n"
               (- defined pending)
               defined
               failed
-              duration)
-           (buttercup--print "Ran %s specs, %s failed, in %.1f seconds.\n"
+              (seconds-to-string duration))
+           (buttercup--print "Ran %s specs, %s failed, in %s.\n"
                              defined
                              failed
-                             duration))))
+                             (seconds-to-string duration)))))
 
       (_
        (error "Unknown event %s" event)))))
@@ -1559,32 +1659,36 @@ colors.
 
 EVENT and ARG are described in `buttercup-reporter'."
   (pcase event
+    (`spec-started
+     (unless (string-match-p "[\n\v\f]" (buttercup-spec-description arg))
+       (buttercup-reporter-batch event arg)))
     (`spec-done
-     (let ((level (length (buttercup-suite-or-spec-parents arg))))
-       (cond
-        ((eq (buttercup-spec-status arg) 'passed)
-         (buttercup--print (buttercup-colorize "\r%s%s" 'green)
-                           (make-string (* 2 level) ?\s)
-                           (buttercup-spec-description arg)))
-        ((eq (buttercup-spec-status arg) 'failed)
-         (buttercup--print (buttercup-colorize "\r%s%s  FAILED" 'red)
-                           (make-string (* 2 level) ?\s)
-                           (buttercup-spec-description arg))
-         (setq buttercup-reporter-batch--failures
-               (append buttercup-reporter-batch--failures
-                       (list arg))))
-        ((eq (buttercup-spec-status arg) 'pending)
-         (if (equal (buttercup-spec-failure-description arg) "SKIPPED")
-             (buttercup--print "  %s" (buttercup-spec-failure-description arg))
-           (buttercup--print (buttercup-colorize "\r%s%s  %s" 'yellow)
-                             (make-string (* 2 level) ?\s)
-                             (buttercup-spec-description arg)
-                             (buttercup-spec-failure-description arg))))
-        (t
-         (error "Unknown spec status %s" (buttercup-spec-status arg))))
-       (buttercup--print " (%s)\n"
-                         (seconds-to-string
-                          (float-time (buttercup-elapsed-time arg))))))
+     ;; Carriage returns (\r) should not be colorized. It would mess
+     ;; up color handling in Emacs compilation buffers using
+     ;; `ansi-color-apply-on-region' in `compilation-filter-hook'.
+     (pcase (buttercup-spec-status arg)
+       (`passed
+        (buttercup--print
+         "\r%s" (buttercup-colorize (buttercup--indented-description arg) 'green)))
+       (`failed
+        (buttercup--print
+          "\r%s" (buttercup-colorize
+                  (concat (buttercup--indented-description arg) "  FAILED")
+                  'red))
+        (setq buttercup-reporter-batch--failures
+              (append buttercup-reporter-batch--failures
+                      (list arg))))
+       (`pending
+        (if (equal (buttercup-spec-failure-description arg) "SKIPPED")
+            (buttercup--print "  %s" (buttercup-spec-failure-description arg))
+          (buttercup--print
+           "\r%s" (buttercup-colorize
+                   (concat (buttercup--indented-description arg) "  "
+                           (buttercup-spec-failure-description arg))
+                   'yellow))))
+       (_
+        (error "Unknown spec status %s" (buttercup-spec-status arg))))
+     (buttercup--print " (%s)\n" (buttercup-elapsed-time-string arg)))
 
     (`buttercup-done
      (dolist (failed buttercup-reporter-batch--failures)
@@ -1611,26 +1715,26 @@ EVENT and ARG are described in `buttercup-reporter'."
      (let ((defined (buttercup-suites-total-specs-defined arg))
            (pending (buttercup-suites-total-specs-pending arg))
            (failed (buttercup-suites-total-specs-failed arg))
-           (duration (- (float-time)
-                        buttercup-reporter-batch--start-time)))
+           (duration (float-time (time-subtract (current-time)
+                                                buttercup-reporter-batch--start-time))))
        (if (> pending 0)
            (buttercup--print
             (concat
              "Ran %s out of %s specs,"
              (buttercup-colorize " %s failed" (if (eq 0 failed) 'green 'red))
-             ", in %.1f seconds.\n")
+             ", in %s.\n")
             (- defined pending)
             defined
             failed
-            duration)
+            (seconds-to-string duration))
          (buttercup--print
           (concat
            "Ran %s specs,"
            (buttercup-colorize " %s failed" (if (eq 0 failed) 'green 'red))
-           ", in %.1f seconds.\n")
+           ", in %s.\n")
           defined
           failed
-          duration))))
+          (seconds-to-string duration)))))
 
     (_
      ;; Fall through to buttercup-reporter-batch implementation.
@@ -1645,7 +1749,7 @@ FMT and ARGS are passed to `format'."
 
 
 (defadvice display-warning (around buttercup-defer-warnings activate)
-  "Log all warnings to a special buffer while running buttercup tests.
+  "Log all warnings to a special buffer while running buttercup specs.
 
 Emacs' normal display logic for warnings doesn't mix well with
 buttercup, for several reasons. So instead, while a buttercup
@@ -1693,7 +1797,7 @@ the capturing behavior."
 (defun buttercup-colorize (string color)
   "Format STRING with COLOR."
   (let ((color-code (cdr (assoc color buttercup-colors))))
-    (format "\u001b[%sm%s\u001b[0m" color-code string)))
+    (format "\e[%sm%s\e[0m" color-code string)))
 
 (defun buttercup-reporter-interactive (event arg)
   "Reporter for interactive sessions.
