@@ -25,6 +25,7 @@
 
 (require 'buttercup)
 (require 'autoload)
+(require 'ansi-color)
 (require 'ert)
 (require 'cl-lib)
 
@@ -34,6 +35,64 @@
             (lambda () item))
           items))
 
+(defmacro with-local-buttercup (&rest body)
+  "Execute BODY with local buttercup state variables."
+  (declare (debug t) (indent defun))
+  `(let (buttercup--after-all
+         buttercup--after-each
+         buttercup--before-all
+         buttercup--before-each
+         (buttercup--cleanup-functions :invalid)
+         buttercup--current-suite
+         (buttercup-reporter #'ignore)
+         buttercup-suites
+         (buttercup-warning-buffer-name " *ignored buttercup warnings*"))
+     ,@body))
+
+(defun send-string-to-ansi-buffer (buffer string)
+  "A `send-string-to-terminal' variant that sends STRING to BUFFER.
+Any backspace, tab, newline, vertical tab, formfeed, or carriage
+return in STRING will be translared to in-buffer movement to
+emulate a terminal. Escape sequences in STRING are translated to
+text properties using `ansi-color-apply'."
+  (setq string (ansi-color-apply string))
+  (cl-labels ((insert-owrt (text)
+                 "Insert TEXT by first overwriting until end of line."
+                 ;; Delete and insert separately. Otherwise characters
+                 ;; with text properties may remain when the new and
+                 ;; the old text share substrings.
+                 (delete-region (point) ; only delete up to the end of line
+                                (min (+ (point) (length text))
+                                     (line-end-position)))
+                 (insert text))
+              (line-feed ()
+                 "Go to beginning of next line, creating it if necessary."
+                  (end-of-line)
+                  (or (zerop (forward-line))
+                      (insert-and-inherit "\n"))))
+    (with-current-buffer buffer
+      (let ((tab-width 8)          ; terminal uses 8 char tabs
+            (indent-tabs-mode nil) ; make sure move-* does not insert tabs
+            ;; default tab-stops (8 char interval)  Can be nil from 24.4 forward...
+            (tab-stop-list (eval (car (get 'tab-stop-list 'standard-value))))
+            ctrl-char)
+        (save-match-data
+          (while (string-match "\\(.*?\\)\\([\b\t\n\v\f\r]\\)\\([^z-a]*\\)" string)
+            (insert-owrt (match-string 1 string))
+            (setq ctrl-char (aref (match-string 2 string) 0)
+                  string (match-string 3 string))
+            (cl-case ctrl-char
+              (?\b (unless (bolp) (backward-char)))
+              (?\t (move-to-tab-stop))
+              (?\n (line-feed))
+              ((?\v ?\f) (let ((line-pos (current-column)))
+                           (line-feed)
+                           (move-to-column line-pos t)))
+              (?\r (forward-line 0))))
+          ;; print remaining text
+          (insert-owrt string))))))
+
+
 ;;;;;;;;;;
 ;;; expect
 
@@ -84,8 +143,8 @@
               'buttercup-failed)))
 
   (describe "with a matcher argument"
-    (buttercup-define-matcher :always-true (a) t)
-    (buttercup-define-matcher :always-false (a) nil)
+    (buttercup-define-matcher :always-true (_a) t)
+    (buttercup-define-matcher :always-false (_a) nil)
 
     (it "should not raise an error if the matcher returns true"
       (expect (buttercup-expect (lambda () 1) :always-true)
@@ -156,8 +215,7 @@
 ;;;;;;;;;;;;;;;;;;;;;
 ;;; Built-in matchers
 
-;; Are tested in README.md
-
+;; Are tested in docs/writing-tests.md
 
 (buttercup-define-matcher-for-unary-function :test-to-be-truthy identity)
 
@@ -179,24 +237,26 @@
 ;;; Suite and spec data structures
 
 (describe "The `buttercup-suite-add-child' function"
-  (it "should add an element at the end of the list"
+  (it "should add an element at the end of the list and return it"
     (let* ((specs (list (make-buttercup-spec)
                         (make-buttercup-spec)
                         (make-buttercup-spec)))
            (suite (make-buttercup-suite :children specs))
            (spec (make-buttercup-spec)))
 
-      (buttercup-suite-add-child suite spec)
+      (expect (buttercup-suite-add-child suite spec)
+              :to-be spec)
 
       (expect (buttercup-suite-children suite)
               :to-equal
               (append specs (list spec)))))
 
-  (it "should add an element even if the list is empty"
+  (it "should add an element even if the list is empty and return it"
     (let ((suite (make-buttercup-suite :children nil))
           (spec (make-buttercup-spec)))
 
-      (buttercup-suite-add-child suite spec)
+      (expect (buttercup-suite-add-child suite spec)
+              :to-be spec)
 
       (expect (buttercup-suite-children suite)
               :to-equal
@@ -250,6 +310,28 @@
               :to-equal
               2))))
 
+(describe "The `buttercup-suites-total-specs-pending' function"
+  :var (suites)
+  (before-each
+    (with-local-buttercup
+      (describe "first suite"
+        (it "active test" (expect 1 :to-equal 1))
+        (xit "pending test"))
+      (xdescribe "second suite"
+        (it "forced pending" (expect 1 :to-equal 2)))
+      (describe "third suite"
+        (it "potentially skipped" (expect 1 :to-equal 1)))
+      (setq suites buttercup-suites)))
+  (it "should return the number of pending specs in a list of suites"
+    (with-local-buttercup
+      (expect (buttercup-suites-total-specs-pending suites)
+              :to-equal 2)))
+  (it "should also count skipped specs"
+    (with-local-buttercup
+      (buttercup--mark-skipped suites (list "skipped"))
+      (expect (buttercup-suites-total-specs-pending suites)
+              :to-equal 3))))
+
 (describe "The `buttercup-suites-total-specs-failed' function"
   (it "should return the number of failed specs in a list of suites"
     (let ((su1 (make-buttercup-suite :description "su1"))
@@ -322,20 +404,6 @@
         (expect (buttercup-elapsed-time spec)
                 :to-equal (seconds-to-time 1.5))))))
 
-(defmacro with-local-buttercup (&rest body)
-  "Execute BODY with local buttercup state variables."
-  (declare (debug t) (indent defun))
-  `(let (buttercup--after-all
-         buttercup--after-each
-         buttercup--before-all
-         buttercup--before-each
-         buttercup--cleanup-functions
-         buttercup--current-suite
-         (buttercup-reporter #'ignore)
-         buttercup-suites
-         (buttercup-warning-buffer-name " *ignored buttercup warnings*"))
-     ,@body))
-
 (describe "The `buttercup--run-suite' function"
   (before-each
     (spy-on 'buttercup--set-start-time :and-call-through)
@@ -364,7 +432,46 @@
                   :not :to-be nil)
           (expect 'buttercup--set-end-time :to-have-been-called-times 1)
           (expect (buttercup-suite-or-spec-time-ended spec)
-                  :not :to-be nil)))))
+                  :not :to-be nil))))
+
+  (it "should not overwrite pending status with `after-each' results"
+    (with-local-buttercup
+      (let ((suite (make-buttercup-suite))
+            spec)
+        (let ((buttercup--current-suite suite))
+          (after-each (ignore))
+          (setq spec (xit "pending")))
+        (buttercup--run-suite suite)
+        (expect (buttercup-spec-status spec) :to-be 'pending))))
+
+  (describe "should set status to pending"
+    (it "for assume in `before-each'"
+      (with-local-buttercup
+        (describe "suite"
+          (before-each (assume nil "assume nil in before-each"))
+          (it "spec" (expect 1 :to-equal 1))
+          (after-each (ignore)))
+        (buttercup-run)
+        (expect (buttercup-suites-total-specs-pending buttercup-suites)
+                :to-equal 1)))
+    (it "for assume in spec"
+      (with-local-buttercup
+        (describe "suite"
+          (before-each (ignore))
+          (it "spec" (assume nil "assume nil in spec"))
+          (after-each (ignore)))
+        (buttercup-run)
+        (expect (buttercup-suites-total-specs-pending buttercup-suites)
+                :to-equal 1)))
+    (it "for assume in `after-each'"
+      (with-local-buttercup
+        (describe "suite"
+          (before-each (ignore))
+          (it "spec" (expect 1 :to-equal 1))
+          (after-each (assume nil "assume nil in after-each")))
+        (buttercup-run)
+        (expect (buttercup-suites-total-specs-pending buttercup-suites)
+                :to-equal 1)))))
 
 ;;;;;;;;;;;;;;;;;;;;
 ;;; Suites: describe
@@ -444,11 +551,12 @@
               (buttercup-it "" (lambda ())))
             :to-throw))
 
-  (it "should add a spec to the current suite"
+  (it "should add a spec to the current suite and return the spec"
     (let ((buttercup--current-suite (make-buttercup-suite)))
-      (buttercup-it "the test spec"
-        (lambda () 23))
-      (let ((spec (car (buttercup-suite-children buttercup--current-suite))))
+      (let* ((created (buttercup-it "the test spec"
+                        (lambda () 23)))
+             (spec (car (buttercup-suite-children buttercup--current-suite))))
+        (expect created :to-be spec)
         (expect (buttercup-spec-description spec)
                 :to-equal
                 "the test spec")
@@ -602,18 +710,19 @@
               :to-be 'pending)))
 
   (it "should set the failure description to PENDING"
-    (let ((buttercup--current-suite (make-buttercup-suite))
-          spec)
-      (buttercup-xit "bla bla")
-      (setq spec (car (buttercup-suite-children buttercup--current-suite)))
+    (let* ((buttercup--current-suite (make-buttercup-suite))
+           (spec (buttercup-xit "bla bla")))
       (buttercup--update-with-funcall spec (buttercup-spec-function spec))
-      (expect (buttercup-suite-or-spec-failure-description spec) :to-equal "PENDING"))))
+      (expect (buttercup-suite-or-spec-failure-description spec) :to-equal "PENDING")
+      (expect (buttercup-suite-or-spec-status spec) :to-equal 'pending))))
 
 ;;;;;;;;;
 ;;; Spies
 
 (describe "The Spy "
-  (let (saved-test-function saved-test-command)
+  (let (saved-test-function
+        saved-test-command
+        saved-test-function-throws-on-negative)
     ;; We use `before-all' here because some tests need to access the
     ;; same function as previous tests in order to work, so overriding
     ;; the function before each test would invalidate those tests.
@@ -621,19 +730,28 @@
       (setq saved-test-function (and (fboundp 'test-function)
                                      (symbol-function 'test-function))
             saved-test-command (and (fboundp 'test-command)
-                                    (symbol-function 'test-command)))
+                                    (symbol-function 'test-command))
+            saved-test-function-throws-on-negative
+            (and (fboundp 'test-function-throws-on-negative)
+                 (symbol-function 'test-function-throws-on-negative)))
       (fset 'test-function (lambda (a b)
                              (+ a b)))
       (fset 'test-command (lambda ()
                             (interactive)
-                            t)))
+                            t))
+      (fset 'test-function-throws-on-negative
+            (lambda (x) (if (>= x 0) x (error "x is less than zero")))))
     (after-all
       (if saved-test-function
           (fset 'test-function saved-test-function)
         (fmakunbound 'test-function))
       (if saved-test-command
           (fset 'test-command saved-test-command)
-        (fmakunbound 'test-command)))
+        (fmakunbound 'test-command))
+      (if saved-test-function-throws-on-negative
+          (fset 'test-function-throws-on-negative
+                saved-test-function-throws-on-negative)
+        (fmakunbound 'test-function-throws-on-negative)))
 
     (describe "`spy-on' function"
       (it "replaces a symbol's function slot"
@@ -673,6 +791,11 @@
                         :to-be :loaded-successfully))
             (delete-file function-file nil))))
 
+      (it "can spy on non-existing functions"
+        (spy-on 'local-function)
+        (local-function)
+        (expect 'local-function :to-have-been-called))
+
       (it "only accepts ARG for keywords that use it"
         (expect
          (spy-on 'test-function :and-call-through :arg-not-allowed)
@@ -685,7 +808,20 @@
          :not :to-throw)
         (expect
          (test-function 1 2)
-         :to-throw 'error)))
+         :to-throw 'error))
+
+      (describe "will signal en error if"
+        (it "used in before-all"
+          (with-local-buttercup
+            (let ((suite (describe "A bad spy scope"
+                           (before-all
+                             (spy-on 'some-function)))))
+              (expect (run--suite suite)
+                      :to-throw))))
+        (it "used directly in describe"
+          (with-local-buttercup
+            (expect (describe "Not in describe"
+                      (spy-on 'foo)) :to-throw)))))
 
     (describe ":to-have-been-called matcher"
       (before-each
@@ -862,7 +998,7 @@
 
     (describe ":and-call-fake keyword functionality"
       (before-each
-        (spy-on 'test-function :and-call-fake (lambda (a b) 1001)))
+        (spy-on 'test-function :and-call-fake (lambda (_a _b) 1001)))
 
       (it "tracks calls to the function"
         (test-function 42 23)
@@ -881,129 +1017,297 @@
       (it "throws an error when called"
         (expect (test-function 1 2)
                 :to-throw
-                'error "Stubbed error")))))
+                'error "Stubbed error")))
+
+    (describe "error-recording functionality"
+      (before-each
+        (spy-on 'test-function-throws-on-negative :and-call-through))
+
+      (it "records the function as called even if it throws an error"
+        (expect (test-function-throws-on-negative -5) :to-throw)
+        (expect (buttercup--apply-matcher
+                 :to-have-been-called
+                 (list (lambda () 'test-function-throws-on-negative)))
+                :to-be
+                t))
+
+      (it "counts both successful calls and calls that threw errors"
+        (test-function-throws-on-negative 5)
+        (expect (test-function-throws-on-negative -5) :to-throw)
+        (expect (buttercup--apply-matcher
+                 :to-have-been-called-times
+                 (make-list-of-closures '(test-function-throws-on-negative 2)))
+                :to-equal t))
+
+      (it "records args to the function whether it throw an error or not"
+        (test-function-throws-on-negative 5)
+        (expect (test-function-throws-on-negative -5) :to-throw)
+        (expect (buttercup--apply-matcher
+                 :to-have-been-called-with
+                 (make-list-of-closures '(test-function-throws-on-negative 5)))
+                :to-be
+                t)
+        (expect (buttercup--apply-matcher
+                 :to-have-been-called-with
+                 (make-list-of-closures '(test-function-throws-on-negative -5)))
+                :to-be
+                t))
+
+      (it "records the signal thrown by a call to the function"
+        (test-function-throws-on-negative 5)
+        (expect (test-function-throws-on-negative -5) :to-throw 'error)
+        (expect (spy-context-thrown-signal
+                 (spy-calls-first 'test-function-throws-on-negative))
+                :to-throw)
+        (expect (spy-context-thrown-signal
+                 (spy-calls-most-recent 'test-function-throws-on-negative))
+                :to-equal '(error "x is less than zero"))))))
 
 ;;;;;;;;;;;;;
 ;;; Reporters
+(buttercup-define-matcher-for-binary-function
+    :to-equal-including-properties equal-including-properties)
 
-(describe "The batch reporter"
+(describe "The batch reporters"
+  :var (print-buffer)
   (let (parent-suite child-suite spec)
     (before-each
       (setq parent-suite (make-buttercup-suite :description "parent-suite")
             child-suite (make-buttercup-suite :description "child-suite")
-            spec (make-buttercup-spec :description "spec"))
+            spec (make-buttercup-spec :description "spec")
+            print-buffer (generate-new-buffer "*btrcp-reporter-test*"))
       (buttercup-suite-add-child parent-suite child-suite)
       (buttercup-suite-add-child child-suite spec)
-      (spy-on 'buttercup--print))
+      (spy-on 'send-string-to-terminal :and-call-fake
+              (apply-partially #'send-string-to-ansi-buffer print-buffer))
+      ;; Convenience function
+      (spy-on 'buttercup-output :and-call-fake
+              (lambda ()
+                "Return the text of print-buffer."
+                (with-current-buffer print-buffer
+                  (buffer-string)))))
+    (after-each
+      (kill-buffer print-buffer)
+      (setq print-buffer nil))
 
     (describe "on the buttercup-started event"
-      (it "should emit the number of specs"
-        (let ((buttercup-reporter-batch--start-time nil)
-              (buttercup-reporter-batch--failures nil))
-          (buttercup-reporter-batch 'buttercup-started (list parent-suite)))
+      :var (skipped
+            ;; Local var for testing. The real variable is used by the
+            ;; reporter attached to the buttercup instance running
+            ;; these tests.
+            buttercup-reporter-batch--start-time)
+      (before-each
+        (setq skipped (make-buttercup-spec :description "skipped" :status 'pending)))
 
-        (expect 'buttercup--print
-                :to-have-been-called-with
-                "Running %s specs.\n\n"
-                1)))
+      (it "should print the number of specs"
+        (let ((buttercup-reporter-batch--failures nil))
+          (buttercup-reporter-batch 'buttercup-started (list parent-suite)))
+        (expect (buttercup-output) :to-equal-including-properties "Running 1 specs.\n\n"))
+
+      (it "should color-print the number of specs with the default color"
+        (let (buttercup-reporter-batch--failures)
+          (buttercup-reporter-batch-color 'buttercup-started (list parent-suite)))
+        (expect (buttercup-output) :to-equal-including-properties "Running 1 specs.\n\n"))
+
+      (it "should print the number of skipped specs"
+        (let ((buttercup-reporter-batch--failures nil))
+          (buttercup-suite-add-child child-suite skipped)
+          (buttercup-reporter-batch 'buttercup-started (list parent-suite)))
+        (expect (buttercup-output) :to-equal-including-properties "Running 1 out of 2 specs.\n\n"))
+
+      (it "should color-print the number of skipped specs with the default color"
+        (let (buttercup-reporter-batch--failures)
+          (buttercup-suite-add-child child-suite skipped)
+          (buttercup-reporter-batch-color 'buttercup-started (list parent-suite)))
+        (expect (buttercup-output) :to-equal-including-properties "Running 1 out of 2 specs.\n\n")))
 
     (describe "on the suite-started event"
       (it "should emit an indented suite description"
         (buttercup-reporter-batch 'suite-started child-suite)
+        (expect (buttercup-output) :to-equal-including-properties "  child-suite\n"))
 
-        (expect 'buttercup--print
-                :to-have-been-called-with
-                "%s%s\n"
-                "  "
-                "child-suite")))
+      (it "should color-print an indented suite description with the default color"
+        (buttercup-reporter-batch-color 'suite-started child-suite)
+        (expect (buttercup-output) :to-equal-including-properties "  child-suite\n")))
 
     (describe "on the spec-started event"
       (it "should emit an indented spec description"
         (buttercup-reporter-batch 'spec-started spec)
+        (expect (buttercup-output) :to-equal-including-properties "    spec"))
 
-        (expect 'buttercup--print
-                :to-have-been-called-with
-                "%s%s"
-                "    "
-                "spec")))
+      (it "should color-print an indented spec description with the default color"
+        (buttercup-reporter-batch-color 'spec-started spec)
+        (expect (buttercup-output) :to-equal-including-properties "    spec")))
 
     (describe "on the spec-done event"
-      (it "should print no status tag for a passed spec"
-        (buttercup--set-start-time spec)
-        (setf (buttercup-spec-failure-description spec) "DONTSHOW")
-        (buttercup--set-end-time spec)
-        (buttercup-reporter-batch 'spec-done spec)
+      (describe "for a passed spec"
+        (before-each
+          (buttercup--set-start-time spec)
+          (setf (buttercup-spec-failure-description spec) "DONTSHOW")
+          (buttercup--set-end-time spec))
 
-        (expect (mapconcat (apply-partially #'apply #'format)
-                           (spy-calls-all-args 'buttercup--print)
-                           "")
-                :to-match "^\\s-*([0-9]+\\.[0-9]+\\(h\\|m\\|m?s\\))\n$"))
+        (it "should print no status tag"
+          (buttercup-reporter-batch 'spec-started spec)
+          (buttercup-reporter-batch 'spec-done spec)
+          (expect (buttercup-output) :to-equal-including-properties
+                  (format "    spec (%s)\n"
+                          (buttercup-elapsed-time-string spec))))
 
-      (it "should say FAILED for a failed spec"
-        (setf (buttercup-spec-status spec) 'failed)
+        (it "should color-print the description in green and no status tag"
+          (buttercup-reporter-batch-color 'spec-started spec)
+          (buttercup-reporter-batch-color 'spec-done spec)
+          (expect (buttercup-output) :to-equal-including-properties
+                  (ansi-color-apply
+                   (format "\e[32m    spec\e[0m (%s)\n"
+                           (buttercup-elapsed-time-string spec)))))
 
-        (let ((buttercup-reporter-batch--failures nil))
-          (buttercup-reporter-batch 'spec-done spec))
+        (it "should print multiline specs cleanly"
+          (setf (buttercup-spec-description spec) "one\ntwo\vthree")
+          (buttercup-reporter-batch 'spec-started spec)
+          (buttercup-reporter-batch 'spec-done spec)
+          (expect (buttercup-output) :to-equal-including-properties
+                  (format "    one\ntwo\n   three (%s)\n"
+                          (buttercup-elapsed-time-string spec))))
 
-        (expect (mapconcat (apply-partially #'apply #'format)
-                           (spy-calls-all-args 'buttercup--print)
-                           "")
-                :to-match "FAILED\\(\\s-+.*\\)?\n$"))
+        (it "should color-print multiline specs cleanly"
+          (setf (buttercup-spec-description spec) "one\ntwo\vthree")
+          (buttercup-reporter-batch-color 'spec-started spec)
+          (buttercup-reporter-batch-color 'spec-done spec)
+          (expect (buttercup-output) :to-equal-including-properties
+                  (ansi-color-apply
+                   (format "\e[32m    one\ntwo\n   three\e[0m (%s)\n"
+                           (buttercup-elapsed-time-string spec))))))
 
-      (it "should output the failure-description for a pending spec"
-        (setf (buttercup-spec-status spec) 'pending
-              (buttercup-spec-failure-description spec) "DESCRIPTION")
-        (let ((buttercup-reporter-batch--failures nil))
-          (buttercup-reporter-batch 'spec-done spec))
-        (expect (mapconcat (apply-partially #'apply #'format)
-                           (spy-calls-all-args 'buttercup--print) "")
-                :to-match "DESCRIPTION\\(\\s-+.*\\)?\n$"))
+      (describe "for a failed spec"
+        (before-each
+          (buttercup--set-start-time spec)
+          (setf (buttercup-spec-status spec) 'failed)
+          (buttercup--set-end-time spec))
+
+        (it "should say FAILED"
+          (let ((buttercup-reporter-batch--failures nil))
+            (buttercup-reporter-batch 'spec-started spec)
+            (buttercup-reporter-batch 'spec-done spec))
+          (expect (buttercup-output) :to-equal-including-properties
+                  (format "    spec  FAILED (%s)\n"
+                          (buttercup-elapsed-time-string spec))))
+
+        (it "should color-print the description in red and say FAILED"
+          (let ((buttercup-reporter-batch--failures nil))
+            (buttercup-reporter-batch-color 'spec-started spec)
+            (buttercup-reporter-batch-color 'spec-done spec))
+          (expect (buttercup-output) :to-equal-including-properties
+                  (ansi-color-apply
+                   (format "\e[31m    spec  FAILED\e[0m (%s)\n"
+                           (buttercup-elapsed-time-string spec))))))
+
+      (describe "for a pending spec"
+        (before-each
+          (buttercup--set-start-time spec)
+          (setf (buttercup-spec-status spec) 'pending
+                (buttercup-spec-failure-description spec) "DESCRIPTION")
+          (buttercup--set-end-time spec))
+
+        (it "should output the failure-description"
+          (let ((buttercup-reporter-batch--failures nil))
+            (buttercup-reporter-batch 'spec-started spec)
+            (buttercup-reporter-batch 'spec-done spec))
+          (expect (buttercup-output) :to-equal-including-properties
+                  (format "    spec  DESCRIPTION (%s)\n"
+                          (buttercup-elapsed-time-string spec))))
+
+        (it "should color-print the description and failure-description in yellow"
+          (let ((buttercup-reporter-batch--failures nil))
+            (buttercup-reporter-batch-color 'spec-started spec)
+            (buttercup-reporter-batch-color 'spec-done spec))
+          (expect (buttercup-output) :to-equal-including-properties
+                  (ansi-color-apply
+                   (format "\e[33m    spec  DESCRIPTION\e[0m (%s)\n"
+                        (buttercup-elapsed-time-string spec))))))
 
       (it "should throw an error for an unknown spec status"
         (setf (buttercup-spec-status spec) 'unknown)
-
         (expect (buttercup-reporter-batch 'spec-done spec)
-                :to-throw))
-
-      (it "should print the elapsed time for all specs"
-        (dolist (state '(pending failed passed))
-          (spy-calls-reset 'buttercup--print)
-          (setq spec (make-buttercup-spec :description "spec"
-                                          :parent child-suite
-                                          :status state
-                                          :failure-description ""))
-          (buttercup--set-start-time spec)
-          (buttercup--set-end-time spec)
-          (let ((buttercup-reporter-batch--failures nil))
-            (buttercup-reporter-batch 'spec-done spec))
-
-          (expect (mapconcat (apply-partially #'apply #'format)
-                             (spy-calls-all-args 'buttercup--print)
-                             "")
-                  :to-match " ([0-9]+\\(\\.[0-9]+\\)?\\(h\\|m\\|m?s\\))\n$"))))
+                :to-throw)))
 
     (describe "on the suite-done event"
       (it "should emit a newline at the end of the top-level suite"
         (buttercup-reporter-batch 'suite-done parent-suite)
+        (expect (buttercup-output) :to-equal-including-properties "\n"))
 
-        (expect 'buttercup--print :to-have-been-called-with "\n"))
+      (it "should color-print a newline at the end of the top-level suite"
+        (buttercup-reporter-batch-color 'suite-done parent-suite)
+        (expect (buttercup-output) :to-equal-including-properties "\n"))
 
       (it "should not emit anything at the end of other suites"
         (buttercup-reporter-batch 'suite-done child-suite)
+        (expect (buttercup-output) :to-equal-including-properties ""))
 
-        (expect 'buttercup--print :not :to-have-been-called)))
+      (it "should not color-print anything at the end of other suites"
+        (buttercup-reporter-batch-color 'suite-done child-suite)
+        (expect (buttercup-output) :to-equal-including-properties "")))
 
     (describe "on the buttercup-done event"
-      ;; This is a lie. It should do a ton more stuff. We should test
-      ;; that, too.
-      (it "should handle the end event"
-        (buttercup-reporter-batch 'buttercup-done nil))
+      :var ((buttercup-reporter-batch--start-time (current-time))
+            defined-specs pending-specs failed-specs)
+
+      (before-each
+        (setq defined-specs 10 pending-specs 0 failed-specs 0)
+        (spy-on 'buttercup-suites-total-specs-defined :and-call-fake (lambda (&rest a) defined-specs))
+        (spy-on 'buttercup-suites-total-specs-pending :and-call-fake (lambda (&rest a) pending-specs))
+        (spy-on 'buttercup-suites-total-specs-failed :and-call-fake (lambda (&rest a) failed-specs)))
+
+      (it "should print a summary of run and failing specs"
+        (setq failed-specs 6)
+        (let (buttercup-reporter-batch--failures)
+          (buttercup-reporter-batch 'buttercup-done nil))
+        (expect (buttercup-output) :to-match
+                "Ran 10 specs, 6 failed, in [0-9]+.[0-9]+[mu]?s.\n"))
+
+      (it "should color-print `0 failed' specs in green"
+        (let (buttercup-reporter-batch--failures)
+          (buttercup-reporter-batch-color 'buttercup-done nil))
+        (expect (buttercup-output) :to-match
+                "Ran 10 specs, 0 failed, in [0-9]+.[0-9]+[mu]?s.\n")
+        (expect (substring (buttercup-output) 0 (length "Ran 10 specs, 0 failed, in"))
+                :to-equal-including-properties
+                (ansi-color-apply "Ran 10 specs,\e[32m 0 failed\e[0m, in")))
+
+      (it "should color-print `X failed' specs in red"
+        (setq failed-specs 6)
+        (let (buttercup-reporter-batch--failures)
+          (buttercup-reporter-batch-color 'buttercup-done nil))
+        (expect (buttercup-output) :to-match
+                "Ran 10 specs, 6 failed, in [0-9]+.[0-9]+[mu]?s.\n")
+        (expect (substring (buttercup-output) 0 (length "Ran 10 specs, 6 failed, in"))
+                :to-equal-including-properties
+                (ansi-color-apply "Ran 10 specs,\e[31m 6 failed\e[0m, in")))
+
+      (it "should print a summary separating run and pending specs"
+        (setq pending-specs 3)
+        (let (buttercup-reporter-batch--failures)
+          (buttercup-reporter-batch 'buttercup-done nil))
+        (expect (buttercup-output) :to-match
+                "Ran 7 out of 10 specs, 0 failed, in [0-9]+.[0-9]+[mu]?s.\n"))
+
+      (it "should color-print pending spec count in default color"
+        (setq pending-specs 3)
+        (let (buttercup-reporter-batch--failures)
+          (buttercup-reporter-batch 'buttercup-done nil))
+        (expect (buttercup-output) :to-match
+                "Ran 7 out of 10 specs, 0 failed, in [0-9]+.[0-9]+[mu]?s.\n")
+        (expect (substring (buttercup-output)
+                           0 (length "Ran 7 out of 10 specs, 0 failed, in"))
+                :to-equal-including-properties
+                "Ran 7 out of 10 specs, 0 failed, in"))
 
       (it "should not raise any error even if a spec failed"
         (setf (buttercup-spec-status spec) 'failed)
-
-        (expect (buttercup-reporter-batch 'buttercup-done (list spec))
-                :not :to-throw)))
+        (let (buttercup-reporter-batch--failures)
+          (expect (buttercup-reporter-batch 'buttercup-done (list spec))
+                  :not :to-throw)))
+      ;; TODO: Backtrace tests
+      )
 
     (describe "on an unknown event"
       (it "should raise an error"
@@ -1036,7 +1340,7 @@
         (expect 'reporter :to-have-been-called-with 'buttercup-started buttercup-suites)
         (expect 'reporter :to-have-been-called-with 'buttercup-done buttercup-suites)))
     )
-  (it "should call `buttercup--run-suite once per suite"
+  (it "should call `buttercup--run-suite' once per suite"
     (let ((buttercup-suites (list parent-suite)) runner)
       (ignore runner)
       (setf (symbol-function 'runner) (lambda (suite) (ignore suite)))
@@ -1058,6 +1362,47 @@
             :to-have-been-called-with
             "Hello, world")))
 
+(describe "The `buttercup--mark-skipped' function"
+  :var (suites)
+  (before-each
+    (with-local-buttercup
+      (describe "first suite"
+        (describe "inner suite"
+          (it "1-1-1 spec" (ignore))
+          (it "1-1-2 spec" (ignore))
+          (it "1-1-3 spec" (ignore))
+          (it "1-1-4 spec" (ignore))
+          (it "1-1-5 spec" (ignore))
+          (xit "1-1-6 spec" (ignore)))
+        (it "1-1 spec" (ignore)))
+      (xdescribe "second suite"
+        (it "2-1 spec" (ignore))
+        (it "2-2 spec" (ignore))
+        (it "2-3 spec" (ignore))
+        (it "2-4 spec" (ignore)))
+      (setq suites buttercup-suites)))
+  (it "should do nothing with a match-all pattern"
+    (expect (buttercup-suites-total-specs-defined suites) :to-equal 11)
+    (expect (buttercup-suites-total-specs-pending suites) :to-equal 5)
+    (buttercup--mark-skipped suites '("."))
+    (expect (buttercup-suites-total-specs-defined suites) :to-equal 11)
+    (expect (buttercup-suites-total-specs-pending suites) :to-equal 5)
+    (with-local-buttercup
+      (setq buttercup-suites suites)
+      (buttercup-run))
+    (expect (buttercup-suites-total-specs-pending suites) :to-equal 5)
+    (expect (cl-count "SKIPPED" (buttercup--specs suites)
+                      :key #'buttercup-spec-failure-description)
+            :to-equal 0))
+  (it "should mark all specs as pending with no pattern"
+    (buttercup--mark-skipped suites '())
+    (expect (buttercup-suites-total-specs-defined suites) :to-equal 11)
+    (expect (buttercup-suites-total-specs-pending suites) :to-equal 11))
+  (it "should handle multiple patterns"
+    (buttercup--mark-skipped suites '("1-1-[1-2]" "[12]-4"))
+    (expect (buttercup-suites-total-specs-defined suites) :to-equal 11)
+    (expect (buttercup-suites-total-specs-pending suites) :to-equal 8)))
+
 ;;;;;;;;;;;;;;;;;;;;;
 ;;; ERT Compatibility
 
